# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit."""

from __future__ import unicode_literals

MESSAGES = {
    "%d min remaining to read": "剩余 %d 分钟去阅读",
    "(active)": "(活跃)",
    "Also available in:": "其他语言版本：",
    "Archive": "文章存档",
    "Atom feed": "",
    "Authors": "作者",
    "Categories": "分类",
    "Comments": "注释",
    "LANGUAGE": "简体中文",
    "Languages:": "语言：",
    "More posts about %s": "更多 %s 相关文章",
    "Newer posts": "新一篇",
    "Next post": "后一篇",
    "Next": "",
    "No posts found.": "没有找到文章",
    "Nothing found.": "没有找到。",
    "Older posts": "旧一篇",
    "Original site": "原文地址",
    "Posted:": "发表于：",
    "Posts about %s": "文章分类：%s",
    "Posts by %s": "%s 发布",
    "Posts for year %s": "%s年文章",
    "Posts for {month} {day}, {year}": "{year}年{month}月{day}日文章",
    "Posts for {month} {year}": "{year}年{month}月文章",
    "Previous post": "前一篇",
    "Previous": "",
    "Publication date": "发布日期",
    "RSS feed": "RSS 源",
    "Read in English": "中文版",
    "Read more": "更多",
    "Skip to main content": "跳到主内容",
    "Source": "源代码",
    "Subcategories:": "子类别：",
    "Tags and Categories": "标签和分类",
    "Tags": "标签",
    "Toggle navigation": "",
    "Uncategorized": "未分类",
    "Up": "",
    "Updates": "更新",
    "Write your page here.": "在这里书写你的页面。",
    "Write your post here.": "在这里书写你的文章。",
    "old posts, page %d": "旧文章页 %d",
    "page %d": "第 %d 页",
    "{month} {day}, {year}": "",
    "{month} {year}": "",
}
